#!/usr/bin/env python3
from pwn import remote
from Crypto.Util.number import long_to_bytes, inverse
import sympy
import re

def main():
    # Connect to the remote service
    host = "verbal-sleep.picoctf.net"
    port = 58750
    r = remote(host, port)
    
    # Receive all the output
    data = r.recvall(timeout=10).decode()
    
    # Use regex to extract N, e, and ciphertext
    N_match = re.search(r"N:\s*(\d+)", data)
    e_match = re.search(r"e:\s*(\d+)", data)
    c_match = re.search(r"cyphertext:\s*(\d+)", data)
    
    if not (N_match and e_match and c_match):
        print("[-] Error extracting parameters!")
        return

    N = int(N_match.group(1))
    e = int(e_match.group(1))
    ciphertext = int(c_match.group(1))
    
    # Factor N: assuming N is a product of two primes
    factors = sympy.factorint(N)
    if len(factors) != 2:
        print("[-] Error: N does not have exactly two prime factors!")
        return
    
    p, q = list(factors.keys())
    
    # Calculate φ(N)
    phi = (p - 1) * (q - 1)
    
    # Calculate the private key d
    d = inverse(e, phi)
    
    # Decrypt the ciphertext
    m = pow(ciphertext, d, N)
    flag = long_to_bytes(m)
    print("\n[*] FLAG:", flag.decode())

if __name__ == "__main__":
    main()

